Program Beacon;

#pragma CLOCK_FREQ 3579545                        // Change to match xtal frequency

ASM __config _XT_OSC & _WDT_OFF & _PWRTE_OFF & _CP_OFF

// Pin assignment
//   Port A
//     0 : PTT
//     1 : Key1
//     2 : Key2
//     3 : Key3
//     4 : Key4
//
//   Port B                 DIP             DIP On          DIP Off
//     0 : Duty Cycle       4               0 = 50%         1 = 100%
//     1 : Locator          3               0 = JO65HP      1 = HP15EO
//     2 : Send Locator     2               0 = No          1 = Yes
//     3 : Call             1               0 = OZ/OX4MB    1 = OX4MB
//     4 : Last Letter ID   5               0 = Yes         1 = No      (Overrides Duty Cycle)
//     5 : QSK (Not impl.)  6               0 = Yes         1 = No
//     6 :                  7
//     7 :                  8 

Const
  KeyDuration=10;                                 // Duration of key down [s], max 25 s!
  PTTPause   =50;                                 // Pause between PTT and Key [ms]
  DitDuration=100;                                // Duration of a dit [ms], 100 ms is equal to 60 LPM
  
Var
  I:char;
  
Procedure SendDit;

begin
  Output_port_A(11111b);
  Delay_ms(DitDuration);
  Output_port_A(00001b);
  Delay_ms(DitDuration);
end;

Procedure SendDash;

begin
  Output_port_A(11111b);
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
  Output_port_A(00001b);
  Delay_ms(DitDuration);
end;

Procedure CharacterPause;

begin                                             // Pause is actually 3 x dit but last dit in Send procs.
  Delay_ms(DitDuration);
  Delay_ms(DitDuration);
end;

Begin
  Disable_Interrupt('GIE');                       // Initialization of registers
  Clear_Bit('OPTION_REG',7);                      // Enable RB pull up resistors
  Set_Tris_A(00000b);
  Set_Tris_B(11111111b);
  Output_port_A(0);                               // Set all outputs low

  Output_high_port_A(0);                          // PTT On
  Delay_ms(PTTPause);                                    
  
  While 1 do
  begin
    If Input_pin_port_B(4)=1 then                 // Duty cycle 1 = No Last Letter ID, 0 = Last Letter ID
    begin
      If Input_pin_port_B(0)=1 then               // Duty cycle 1 = 100%, 0 = 50%
      begin
        Output_port_A(11111b);                    // Key On    
        Delay_s(KeyDuration);                     // Send carrier
        Output_port_A(00001b);                    // Stop sending carrier
      end;
      else
      begin
        For I:=1 to 10*KeyDuration do
        begin
          Output_port_A(11111b);                  // Key On
          Delay_ms(50);                           // Send carrier
          Output_port_A(00001b);                  // Stop sending carrier
          Delay_ms(50);                           // 
        end;  
      end;
    end;
    else
    begin
      For I:=1 to 15 do
      begin
        If Input_pin_port_B(3)=1 then             // Call 0 or 1 = OX4MB
        begin
          SendDash;                               // Send B, -***
          SendDit;
          SendDit;
          SendDit;
        end;
        else
        begin
          SendDash;                               // Send B, -***
          SendDit;
          SendDit;
          SendDit;
        end;

        CharacterPause;                           // Inter character pause
        CharacterPause;                           // Inter character pause
      end;
    end;

    Delay_ms(250);                                // 500 ms pause between keying and ID
    Delay_ms(250);

    If Input_pin_port_B(3)=1 then                 // If 0 then call = OX4MB
    begin
      SendDash;                                   // Send O, ---
      SendDash;
      SendDash;
    
      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send X, -**-
      SendDit;
      SendDit;
      SendDash;
    
      CharacterPause;                             // Inter character pause
    
      SendDit;                                    // Send 4, ****-
      SendDit;
      SendDit;
      SendDit;
      SendDash;
    
      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send M, --
      SendDash;

      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send B, -***
      SendDit;
      SendDit;
      SendDit;
    end;
    else
    begin
      SendDash;                                   // Send O, ---
      SendDash;
      SendDash;
    
      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send Z, --**
      SendDash;
      SendDit;
      SendDit;

      CharacterPause;                             // Inter character pause

      SendDash;                                   // Send /, -**-*
      SendDit;
      SendDit;
      SendDash;
      SendDit;

      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send O, ---
      SendDash;
      SendDash;
    
      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send X, -**-
      SendDit;
      SendDit;
      SendDash;
    
      CharacterPause;                             // Inter character pause
    
      SendDit;                                    // Send 4, ****-
      SendDit;
      SendDit;
      SendDit;
      SendDash;
    
      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send M, --
      SendDash;

      CharacterPause;                             // Inter character pause
    
      SendDash;                                   // Send B, -***
      SendDit;
      SendDit;
      SendDit;
    end;

    If Input_pin_port_B(2)=1 then                 // Send locator if 1
    begin
      CharacterPause;                             // Word pause, 7 x dit
      CharacterPause;                             
      CharacterPause;                             
      
      If Input_pin_port_B(1)=1 then               // Send HP15EO if 1, GP47TA if 0
      begin
        SendDit;                                  // Send H, ****
        SendDit; 
        SendDit; 
        SendDit; 

        CharacterPause;                           // Inter character pause
            
        SendDit;                                  // Send P, *--*
        SendDash;
        SendDash;
        SendDit; 
    
        CharacterPause;                           // Inter character pause

        SendDit;                                  // Send 1, *----
        SendDash;
        SendDash;
        SendDash;
        SendDash;
              
        CharacterPause;                           // Inter character pause

        SendDit;                                  // Send 5, *****
        SendDit;
        SendDit;
        SendDit;
        SendDit;
    
        CharacterPause;                           // Inter character pause
        
        SendDit;                                  // Send E, *
    
        CharacterPause;                           // Inter character pause
    
        SendDash;                                 // Send O, ---
        SendDash;
        SendDash;
      end;
      else
      begin
        SendDit;                                  // Send J, *---
        SendDash;
        SendDash;
        SendDash;

        CharacterPause;                           // Inter character pause
            
        SendDash;                                 // Send O, ---
        SendDash;
        SendDash;

        CharacterPause;                           // Inter character pause

        SendDash;                                 // Send 6, -****
        SendDit;
        SendDit;
        SendDit;
        SendDit;
    
        CharacterPause;                           // Inter character pause

        SendDit;                                  // Send 5, *****
        SendDit;
        SendDit;
        SendDit;
        SendDit;
   
        CharacterPause;                           // Inter character pause
        
        SendDit;                                  // Send H, *****
        SendDit;
        SendDit;
        SendDit;
    
        CharacterPause;                           // Inter character pause
    
        SendDit;                                  // Send P, *--*
        SendDash;
        SendDash;
        SendDit;
      end;
    end;

    Delay_ms(250);                                // 500 ms pause between ID and keying
    Delay_ms(250);
  end;
End.